////////////////////////////////////////////////////////////////////////////////
//
// (C) 2010 Sarah Eris Horsley Caffee
//
// mandelbrot - A simple Mandelbrot Set viewer.
//
// Fractal.cpp
// Fractal calculator virtual class implementation.  Actual fractals should 
// implement this.
//
////////////////////////////////////////////////////////////////////////////////

#include <string>

#include "SDL.h"

#include "Fractal.h"


////////////////////////////////////////////////////////////////////////////////

Fractal::Fractal() :
   name ("No fractal selected"),
   re_min (0),
   im_max (0),
   size (0),
   max_iter (0),
   surface (NULL),
   calc_needed (false)
   {
   }

////////////////////////////////////////////////////////////////////////////////
void Fractal::set_re_min(const double r_min)
   {
   re_min = r_min;
   }


////////////////////////////////////////////////////////////////////////////////
void Fractal::set_im_max(const double i_max)
   {
   im_max = i_max;
   }


////////////////////////////////////////////////////////////////////////////////
void Fractal::set_size(const double s)
   {
   if (s > 0.0)
      size = s;
   else
      size = 1.0;
   }


////////////////////////////////////////////////////////////////////////////////
void Fractal::set_max_iter(const unsigned int iter)
   {
   if (iter > 0)
      max_iter = iter;
   else
      max_iter = 1;
   }


////////////////////////////////////////////////////////////////////////////////
void Fractal::inc_max_iter()
   {
   max_iter++;
   }


////////////////////////////////////////////////////////////////////////////////
void Fractal::dec_max_iter()
   {
   if (max_iter > 0)
      max_iter--;
   }


////////////////////////////////////////////////////////////////////////////////
void Fractal::set_calc_needed()
   {
   calc_needed = true;
   }


////////////////////////////////////////////////////////////////////////////////
double Fractal::get_re_min()
   {
   return re_min;
   }


////////////////////////////////////////////////////////////////////////////////
double Fractal::get_im_max()
   {
   return im_max;
   }


////////////////////////////////////////////////////////////////////////////////
double Fractal::get_size()
   {
   return size;
   }


////////////////////////////////////////////////////////////////////////////////
unsigned int Fractal::get_max_iter()
   {
   return max_iter;
   }


////////////////////////////////////////////////////////////////////////////////
const char * Fractal::get_name()
   {
   return name.c_str();
   }

////////////////////////////////////////////////////////////////////////////////
void Fractal::draw_pixel(int x, int y, Uint32 * color)
   {
   static char * data;

   if (SDL_MUSTLOCK(surface)) SDL_LockSurface(surface);

   data = (char *) surface->pixels 
	 + y * surface->pitch
	 + x * surface->format->BytesPerPixel;
   memcpy(data, color, surface->format->BytesPerPixel);

   if (SDL_MUSTLOCK(surface)) SDL_UnlockSurface(surface);
   }

